@@ -6,11 +6,12 @@ import oommfc.scripts
 from .delete import delete
 from .compute import compute
 from .drivers import Driver, MinDriver, TimeDriver, HysteresisDriver
-from .evolvers import CGEvolver, EulerEvolver, RungeKuttaEvolver, \
-    SpinTEvolver, SpinXferEvolver
+from .evolvers import (CGEvolver, EulerEvolver, RungeKuttaEvolver,
+                       SpinTEvolver, SpinXferEvolver, UHH_ThetaEvolver,
+                       Xf_ThermHeunEvolver, Xf_ThermSpinXferEvolver)
 
 __version__ = pkg_resources.get_distribution(__name__).version
-__dependencies__ = pkg_resources.require(__name__)
+runner = oommfc.oommf.oommf.Runner()
 
 
 def test():
