@@ -1,4 +1,5 @@
 import os
+import numpy as np
 import sys
 import abc
 import glob
@@ -29,9 +30,16 @@ class Driver(mm.Driver):
     """Driver base class.
 
     """
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        if hasattr(self, 'evolver'):
+            self.autoselect_evolver = False
+        else:
+            self.autoselect_evolver = True
+
     @abc.abstractmethod
     def _checkargs(self, **kwargs):
-        """Abstract method defined in a derived driver class.
+        """Abstract method for checking arguments.
 
         """
         pass  # pragma: no cover
@@ -143,7 +151,7 @@ class Driver(mm.Driver):
                         numbers = list(zip(*[i.split('-')
                                              for i in compute_dirs]))[1]
                         numbers = list(map(int, numbers))
-                        system.drive_number = max(numbers) + 1
+                        system.compute_number = max(numbers) + 1
                     else:
                         msg = (f'Directory {system.name=} already exists. To '
                                f'append drives to it, pass append=True.')
@@ -163,6 +171,12 @@ class Driver(mm.Driver):
         if not os.path.exists(workingdir):
             os.makedirs(workingdir)
 
+        # compute tlist for time-dependent field (current)
+        for term in system.energy:
+            if (hasattr(term, 'time_dependence')
+                    and callable(term.time_dependence)):
+                self._time_dependence(term=term, **kwargs)
+
         # Change directory to workingdir
         with _changedir(workingdir):
             # Generate the necessary filenames.
@@ -178,6 +192,14 @@ class Driver(mm.Driver):
             with open(miffilename, 'w') as miffile:
                 miffile.write(mif)
 
+            # free memory
+            for term in system.energy:
+                if (hasattr(term, 'time_dependence')
+                        and callable(term.time_dependence)):
+                    del term.tlist
+                    del term.dtlist
+                    del mif
+
             # Generate and save json info file for a drive (not compute).
             if compute is None:
                 info = {}
@@ -193,16 +215,16 @@ class Driver(mm.Driver):
             # Get right OOMMF runner depending on whether there is DMI.
             if runner is None:
                 if sys.platform != 'win32':
-                    runner = oc.oommf.get_oommf_runner()
+                    runner = oc.runner.runner
                 else:
                     if hasattr(system.energy, 'dmi'):
-                        if (system.energy.dmi.crystalclass == 'Cnv' and
-                                system.m.mesh.bc == ''):
-                            runner = oc.oommf.get_oommf_runner()
+                        if (system.energy.dmi.crystalclass in ['Cnv', 'Cnv_z']
+                                and system.m.mesh.bc == ''):
+                            runner = oc.runner.runner
                         else:
                             runner = oc.oommf.DockerOOMMFRunner()
                     else:
-                        runner = oc.oommf.get_oommf_runner()
+                        runner = oc.runner.runner
             runner.call(argstr=miffilename, n_threads=n_threads)
 
             # Update system's m and datatable attributes if the derivation of
@@ -227,3 +249,24 @@ class Driver(mm.Driver):
             system.drive_number += 1
         else:
             system.compute_number += 1
+
+        # remove information about fixed cells for subsequent runs
+        if hasattr(self.evolver, 'fixed_spins'):
+            del self.evolver.fixed_spins
+
+    def _time_dependence(self, term, **kwargs):
+        try:
+            tmax = kwargs['t']
+        except KeyError:
+            msg = (f'Time-dependent term {term.__class__.__name__=} must be '
+                   'used with time driver.')
+            raise RuntimeError(msg)
+        ts = np.arange(0, tmax + term.tstep, term.tstep)
+        try:  # vector output from term.time_dependence
+            tlist = [list(term.time_dependence(t)) for t in ts]
+            dtlist = (np.gradient(tlist)[0] / term.tstep).tolist()
+        except TypeError:
+            tlist = [term.time_dependence(t) for t in ts]
+            dtlist = list(np.gradient(tlist) / term.tstep)
+        term.tlist = tlist
+        term.dtlist = dtlist
