@@ -1,6 +1,7 @@
 import numpy as np
 import oommfc as oc
 import micromagneticmodel as mm
+import ubermagutil.typesystem as ts
 
 
 def driver_script(driver, system, fixed_subregions=None, compute=None,
@@ -100,8 +101,12 @@ def driver_script(driver, system, fixed_subregions=None, compute=None,
 
     if isinstance(driver, oc.TimeDriver):
         # Check evolver and set default if not passed.
-        if not hasattr(driver, 'evolver'):
-            if mm.ZhangLi() in system.dynamics:
+        if driver.autoselect_evolver:
+            if system.T > 0:
+                msg = ('For simulations at finite temperature the evolver must'
+                       'be specified explicitely.')
+                raise RuntimeError(msg)
+            elif mm.ZhangLi() in system.dynamics:
                 driver.evolver = oc.SpinTEvolver()
             elif mm.Slonczewski() in system.dynamics:
                 driver.evolver = oc.SpinXferEvolver()
@@ -110,13 +115,25 @@ def driver_script(driver, system, fixed_subregions=None, compute=None,
         elif not isinstance(driver.evolver, (oc.EulerEvolver,
                                              oc.RungeKuttaEvolver,
                                              oc.SpinTEvolver,
-                                             oc.SpinXferEvolver)):
+                                             oc.SpinXferEvolver,
+                                             oc.UHH_ThetaEvolver,
+                                             oc.Xf_ThermHeunEvolver,
+                                             oc.Xf_ThermSpinXferEvolver,
+                                             )):
             msg = f'Cannot use {type(driver.evolver)} for evolver.'
             raise TypeError(msg)
 
         # Extract dynamics equation parameters.
         if mm.Precession() in system.dynamics:
-            driver.evolver.gamma_G = system.dynamics.precession.gamma0
+            if isinstance(driver.evolver, oc.UHH_ThetaEvolver):
+                pref = 1
+                if mm.Damping() in system.dynamics:
+                    pref += system.dynamics.damping.alpha**2
+                driver.evolver.gamma_LL = (
+                    system.dynamics.precession.gamma0 / pref)
+                driver.evolver.do_precess = 1
+            else:
+                driver.evolver.gamma_G = system.dynamics.precession.gamma0
         else:
             driver.evolver.do_precess = 0
         if mm.Damping() in system.dynamics:
@@ -126,19 +143,44 @@ def driver_script(driver, system, fixed_subregions=None, compute=None,
         if mm.ZhangLi() in system.dynamics:
             driver.evolver.u = system.dynamics.zhangli.u
             driver.evolver.beta = system.dynamics.zhangli.beta
+            for arg in ['time_dependence', 'tstep', 'tcl_strings']:
+                if hasattr(system.dynamics.zhangli, arg):
+                    setattr(driver.evolver, arg,
+                            getattr(system.dynamics.zhangli, arg))
         if mm.Slonczewski() in system.dynamics:
             driver.evolver.J = system.dynamics.slonczewski.J
             driver.evolver.mp = system.dynamics.slonczewski.mp
             driver.evolver.P = system.dynamics.slonczewski.P
             driver.evolver.Lambda = system.dynamics.slonczewski.Lambda
-            driver.evolver.eps_prime = system.dynamics.slonczewski.eps_prime
+            if isinstance(system.dynamics.slonczewski.eps_prime,
+                          ts.Descriptor):
+                driver.evolver.eps_prime = 0
+            else:
+                driver.evolver.eps_prime = \
+                    system.dynamics.slonczewski.eps_prime
+            for arg in ['time_dependence', 'tstep', 'tcl_strings']:
+                if (hasattr(system.dynamics.slonczewski, arg)
+                        and not isinstance(
+                            getattr(system.dynamics.slonczewski, arg),
+                            ts.Descriptor)):
+                    setattr(driver.evolver, arg,
+                            getattr(system.dynamics.slonczewski, arg))
+        if isinstance(driver.evolver, (oc.UHH_ThetaEvolver,
+                                       oc.Xf_ThermHeunEvolver,
+                                       oc.Xf_ThermSpinXferEvolver)):
+            driver.evolver.temperature = system.T
+        else:
+            if system.T > 0:
+                msg = (f'Evolver {driver.evolver} does not support finite'
+                       ' temperature.')
+                raise TypeError(msg)
 
         # Fixed spins
         if fixed_subregions is not None:
             resstr = f'{{main_atlas {" ".join(fixed_subregions)}}}'
             driver.evolver.fixed_spins = resstr
 
-        mif += oc.scripts.evolver_script(driver.evolver)
+        mif += oc.scripts.evolver_script(driver.evolver, **kwargs)
 
         # Extract time and number of steps.
         t, n = kwargs['t'], kwargs['n']
