@@ -13,8 +13,8 @@ def check_runner(runner):
     os.chdir(dirname)
     argstr = 'test_oommf.mif'
     res = runner.call(argstr)
-    version = runner.version()
-    platform = runner.platform()
+    version = runner.version
+    platform = runner.platform
 
     assert isinstance(version, str)
     assert len(version) > 0
@@ -33,7 +33,7 @@ def check_runner(runner):
 @pytest.mark.skip(reason='Temporary')
 def test_tcl_oommf_runner():
     # TclOOMMFRunner runs when OOMMFTCL environment variable is set. On
-    # TravisCI OOMMFTCL environment variable is set inside ubermag/oommf docker
+    # TravisCI OOMMFTCL environment variable is set inside oommf/oommf docker
     # image.
     oommf_tcl = os.environ.get('OOMMFTCL', None)
     runner = oo.TclOOMMFRunner(oommf_tcl)
@@ -44,7 +44,7 @@ def test_tcl_oommf_runner():
 @pytest.mark.skip(reason='Temporary')
 def test_exe_oommf_runner():
     # ExeOOMMFRunner runs when callable OOMMF exists ('oommf'). On TravisCI
-    # oommf is an executable inside ubermag/oommf docker image.
+    # oommf is an executable inside oommf/oommf docker image.
     oommf_exe = 'oommf'
     runner = oo.ExeOOMMFRunner(oommf_exe)
     check_runner(runner)
@@ -59,7 +59,7 @@ def test_docker_oommf_runner():
     # on host or TravisCI. It can be run using make test-docker on host if
     # docker is installed.
     docker_exe = 'docker'
-    image = 'ubermag/oommf'
+    image = 'oommf/oommf'
     runner = oo.DockerOOMMFRunner(docker_exe, image)
     check_runner(runner)
 
@@ -122,11 +122,25 @@ def test_get_oommf_runner():
 
 def test_get_oommf_runner():
     # This is a shorter version of the previous test for testing on host.
-    oommf_runner = oo.get_oommf_runner(use_cache=False)
+    oc.runner.autoselect_runner()
+    oommf_runner = oc.runner.runner
     assert isinstance(oommf_runner, oo.OOMMFRunner)
     check_runner(oommf_runner)
 
 
+@pytest.mark.skip(reason='We need to think about how to test this properly.')
+def test_set_oommf_runner():
+    oc.runner.runner = oo.TclOOMMFRunner()
+    assert isinstance(oc.runner.runner, oo.TclOOMMFRunner)
+
+    if sys.platform != 'win32':
+        oc.runner.runner = oo.ExeOOMMFRunner()
+        assert isinstance(oc.runner.runner, oo.ExeOOMMFRunner)
+
+    oc.runner.runner = oo.DockerOOMMFRunner()
+    assert isinstance(oc.runner.runner, oo.DockerOOMMFRunner)
+
+
 @pytest.mark.skip(reason='Temporary')
 def test_status():
     assert oo.status() == 0
@@ -137,7 +151,8 @@ def test_overhead():
 
 
 def test_runtimeerror():
-    oommf_runner = oo.get_oommf_runner(use_cache=False)
+    oc.runner.autoselect_runner()
+    oommf_runner = oc.runner.runner
     with pytest.raises(RuntimeError):
         oommf_runner.call('+wrong_argstr')
 
